local bdUI, c, l = unpack(select(2, ...))
local mod = bdUI:get_module("Groups")
local config
local oUF = bdUI.oUF
local initialized = false

--======================================================
-- Update the raidframe header with new configuration values
--======================================================
local function update_pet_header()
	if (InCombatLockdown()) then return end

	local header = mod.petframeHeader

	mod:resize_container(mod.petframeHeader, mod.pets_holder, config.pets_width, config.pets_height, 2)

	local group_by, group_sort, sort_method, yOffset, xOffset, new_group_anchor, new_player_anchor, hgrowth, vgrowth, num_groups = mod:get_attributes()

	-- growth/spacing
	header:SetAttribute("columnAnchorPoint", new_group_anchor)
	header:SetAttribute("point", new_player_anchor)

	if (config.group_growth == "Right") then
		header:SetAttribute("columnSpacing", xOffset)
	else
		header:SetAttribute("columnSpacing", -yOffset)
	end

	header:SetAttribute("yOffset", yOffset)
	header:SetAttribute("xOffset", xOffset)
	
	-- when to show
	header:SetAttribute("maxColumns", num_groups)
	
	-- width/height
	header:SetAttribute("initial-width", config.pets_width)
	header:SetAttribute("initial-height", config.pets_height)
	
	-- grouping/sorting
	header:SetAttribute("groupBy", false)
	header:SetAttribute("groupingOrder", group_sort)
	header:SetAttribute("sortMethod", sort_method)
end

local function initialize()
	-- pets
	local pets_holder = CreateFrame('frame', "bdGridPets", UIParent)
	pets_holder:SetSize(config.pets_width, config.pets_height * 5)
	pets_holder:SetPoint("LEFT", mod.raid_holder, "RIGHT", 10, 0)
	bdMove:set_moveable(pets_holder, "Pet Frames")

	-- register events for resizing the box/group size
	pets_holder:SetScript("OnEvent", function(self, event, arg1)
		if (event == "PLAYER_ENTERING_WORLD") then
			C_Timer.After(2, function()
				update_pet_header()
			end)
		else
			update_pet_header()
		end
	end)

	mod.pets_holder = pets_holder

	-- send to factory
	-- oUF:Factory(function(self)
	oUF:SetActiveStyle('bdGrid')

	-- basic information generated by config
	local group_by, group_sort, sort_method, yOffset, xOffset, new_group_anchor, new_player_anchor, hgrowth, vgrowth, num_groups = mod:get_attributes()

	-- ouf gives us a secureheader for attributes
	mod.petframeHeader = oUF:SpawnHeader(nil, "SecureGroupPetHeaderTemplate", 'raid',
		"showParty", false,
		"showSolo", true,
		"showRaid", true,
		"initial-scale", 1,
		"unitsPerColumn", 5,
		"columnSpacing", yOffset,
		"xOffset", xOffset,
		"yOffset", yOffset,
		"maxColumns", num_groups,
		"maxColumns", pets,
		"columnAnchorPoint", new_group_anchor,
		"initial-width", config.pets_width,
		"initial-height", config.pets_height,
		"point", new_player_anchor,
		'oUF-initialConfigFunction', format('self:SetWidth(%d); self:SetHeight(%d);', config.pets_width, config.pets_height)
	)

	update_pet_header()
	-- end)
end

local function callback()
	if (not mod.petframeHeader) then return end
	update_pet_header()
end

local function disable(_config)
	if (not mod.petframeHeader) then return end
	config = _config
	mod.pets_holder:RegisterEvent("PLAYER_REGEN_ENABLED")
	mod.pets_holder:RegisterEvent("PLAYER_ENTERING_WORLD")
	mod.pets_holder:RegisterEvent("RAID_ROSTER_UPDATE")
	mod.pets_holder:RegisterEvent("GROUP_JOINED")
	mod.pets_holder:RegisterEvent("GROUP_ROSTER_UPDATE")
	mod.pets_holder:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	-- mod.raid_holder:UnregisterEvent("ZONE_CHANGED_NEW_AREA")

	mod.pets_holder:Hide()
	mod.petframeHeader:SetAttribute("showParty", false)
	mod.petframeHeader:SetAttribute("showSolo", false)
	mod.petframeHeader:SetAttribute("showRaid", false)

	return false
end

local function enable(_config)
	config = _config

	-- disable it
	if (not config.pets_frame_enable) then
		return false
	end

	-- run first time
	if (not initialized) then
		initialize()
		initialized = true
	end

	-- show the frame
	mod.pets_holder:Show()
	mod.petframeHeader:SetAttribute("showParty", false)
	mod.petframeHeader:SetAttribute("showSolo", true)
	mod.petframeHeader:SetAttribute("showRaid", true)

	-- register callbacks
	mod.pets_holder:RegisterEvent("PLAYER_REGEN_ENABLED")
	mod.pets_holder:RegisterEvent("PLAYER_ENTERING_WORLD")
	mod.pets_holder:RegisterEvent("RAID_ROSTER_UPDATE")
	mod.pets_holder:RegisterEvent("GROUP_JOINED")
	mod.pets_holder:RegisterEvent("GROUP_ROSTER_UPDATE")
	mod.pets_holder:RegisterEvent("ZONE_CHANGED_NEW_AREA")

	-- now run config callback on it
	callback()

	-- it's enabled, don't run to disabled function
	return true
end

local function path() end
mod:add_element('pet_frames', path, enable, disable)